#include "policyd.h"


/*
 *
 *
 *                           Policy Daemon
 *
 *  policy daemon is used in conjuction with postfix to combat spam.
 *
 *  Copyright (C) 2004 Cami Sardinha (cami@mweb.co.za)
 *
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the  GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This program  is  distributed  in the hope that  it will be useful, but
 *  WITHOUT  WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free  Software Foundation Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 */


/*
 * function: throttle_check
 *  purpose: throttle users based on SASL info or envelope FROM
 *   return: 0 for new record, 1 for update
 */
int
throttle_check (unsigned int fd)
{
  unsigned int tnum = 0;

  if(DEBUG > 0)
    logmessage("DEBUG: fd: %d checking throttle\n", fd);
  
  /* build up & execute query */
  if(SENDER_THROTTLE_HOST == 1)
  {
    tnum = 1;
    snprintf(mysqlquery_array[fd], 512,
        "SELECT * FROM throttle WHERE _from='%s'", host_array[fd][2]);
    
  } else if((SENDER_THROTTLE_SASL == 1) && (triplet_array[fd][4][0] != 0x00))  {
    tnum = 2;
    snprintf(mysqlquery_array[fd], 512,
      "SELECT * FROM throttle WHERE _from='%s'", triplet_array[fd][4]);
  } else {
    tnum = 3;
    snprintf(mysqlquery_array[fd], 512,
        "SELECT * FROM throttle WHERE _from='%s'", triplet_array[fd][1]);
  }
  if(db_charquery(fd) == -1) return(db_failure(fd, "throttle"));    
  
  /* max messages is disabled in database, fall back to config default */
  if(atol(mysqlchar_array[fd][1]) == 0)
    snprintf(mysqlchar_array[fd][1], sizeof(mysqlchar_array[fd][1]),
	"%d", SENDERMSGLIMIT);

  /* max user quota is disabled in database, fall back to config defaults */
  if(atol(mysqlchar_array[fd][5]) == 0)
    snprintf(mysqlchar_array[fd][5], sizeof(mysqlchar_array[fd][5]),
	"%d", SENDERQUOTALIMIT);
  
  /* max time limit is disabled in database, fall back to config defaults */
  if(atol(mysqlchar_array[fd][6]) == 0)
    snprintf(mysqlchar_array[fd][6], sizeof(mysqlchar_array[fd][6]),
	"%d", SENDERTIMELIMIT);

  /* max message size is disabled in database, fall back to config defaults */
  if(atol(mysqlchar_array[fd][7]) == 0)
    snprintf(mysqlchar_array[fd][7], sizeof(mysqlchar_array[fd][7]),
	"%d", SENDERMSGSIZE);
  
  /* we selectively choose which throttle module we want */
  switch(tnum)
  {
    case 1:
      return(throttle_host(fd));

    case 2:
      return(throttle_sasl(fd));

    case 3:
      return(throttle_from(fd));

    default:
      logmessage("fatal: throttle_check(): no tnum\n");
      return (-1);
  }
  
  return (0); /* never reached */
}
 
/* EOF */
