#include "policyd.h"


/*
 *
 *
 *                           Policy Daemon
 *
 *  policy daemon is used in conjuction with postfix to combat spam.
 *
 *  Copyright (C) 2004 Cami Sardinha (cami@mweb.co.za)
 *
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the  GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This program  is  distributed  in the hope that  it will be useful, but
 *  WITHOUT  WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free  Software Foundation Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 */


/*
 * function: throttle_host
 *  purpose: throttle users based on host
 *   return: 0 for new record, 1 for update
 */
int
throttle_host (unsigned int fd)
{

  if(DEBUG > 0)
    logmessage("DEBUG: fd: %d checking throttle-host\n", fd);

  /* user is not in the database */
  if(strlen(mysqlchar_array[fd][0]) < 2)
  {
    logmessage("rcpt=%lu, throttle=new, host=%s, from=%s, to=%s, size=%d/%d, quota=%d/%d, count=1/%d(1)\n",
      rcpt_count,			/* recipient count	*/
      host_array[fd][2],		/* host			*/
      triplet_array[fd][1],		/* from			*/
      triplet_array[fd][2],		/* to			*/
      atol(triplet_array[fd][3]),	/* size_cur		*/
      atol(mysqlchar_array[fd][7]),	/* size_max		*/
      atol(triplet_array[fd][3]),	/* quota_cur		*/
      atol(mysqlchar_array[fd][5]),	/* quota_max		*/
      atol(mysqlchar_array[fd][1])	/* count_max		*/
    );

    /* build up & execute query */
    snprintf(mysqlquery_array[fd], 512,
      "INSERT DELAYED INTO throttle (_date,_from,_quota_cur) VALUES (%d,'%s',%d)",
       timenow, host_array[fd][2], atoi(triplet_array[fd][3]));
    if(db_doquery(fd) == -1) return(db_failure(fd, "throttle"));

    /* sender does not exist in the database, insert and allow */
    return (0);
  }

  /* if sender is sending a message bigger than allowed, reject */
  if(atol(triplet_array[fd][3]) >= atol(mysqlchar_array[fd][7]))
  {
    logmessage("rcpt=%lu, throttle=abuse, host=%s, from=%s, to=%s, size=%d/%d, quota=%d/%d, count=%d/%d(%d)\n",
      rcpt_count,                       /* recipient count      */
      host_array[fd][2],                /* host                 */
      triplet_array[fd][1],             /* from                 */
      triplet_array[fd][2],             /* to                   */
      atol(triplet_array[fd][3]),       /* size_cur             */
      atol(mysqlchar_array[fd][7]),     /* size_max             */
      atol(mysqlchar_array[fd][4]),	/* quota_cur		*/
      atol(mysqlchar_array[fd][5]),     /* quota_max            */
      atol(mysqlchar_array[fd][2]),	/* count_cur		*/
      atol(mysqlchar_array[fd][1]),     /* count_max            */
      atol(mysqlchar_array[fd][8])	/* count_tot		*/
    );

    return (-3);
  }

  /* if time has expired, clear quota for size+message count */
  if(timenow > (unsigned int)(atol(mysqlchar_array[fd][6])+atol(mysqlchar_array[fd][3])))
  {
    logmessage("rcpt=%lu, throttle=clear, host=%s, from=%s, to=%s, size=%d/%d, quota=%d/%d, count=0/%d(%d)\n",
      rcpt_count,                       /* recipient count      */
      host_array[fd][2],                /* host                 */
      triplet_array[fd][1],             /* from                 */
      triplet_array[fd][2],             /* to                   */
      atol(triplet_array[fd][3]),       /* size_cur             */
      atol(mysqlchar_array[fd][7]),     /* size_max             */
      atol(mysqlchar_array[fd][4]),     /* quota_cur            */
      atol(mysqlchar_array[fd][5]),     /* quota_max            */
      atol(mysqlchar_array[fd][1]),     /* count_max            */
      atol(mysqlchar_array[fd][8])      /* count_tot            */
    );
      
    /* build up & execute query */
    snprintf(mysqlquery_array[fd], 512,
      "UPDATE throttle SET _count_cur=1,_count_tot=_count_tot+1,_date=%d,_quota_cur=%d WHERE _from='%s'",
      timenow, atoi(triplet_array[fd][3]), host_array[fd][2]);
    if(db_doquery(fd) == -1) return(db_failure(fd, "throttle"));

    /* counter reset because of expiry, allow mail */
    return (0);
  }
  
  /* if the sender is past his quota and the timeout has not expired */
  /* then reject the message */
  if(atol(mysqlchar_array[fd][2]) >= atol(mysqlchar_array[fd][1]) ||
     atol(mysqlchar_array[fd][4]) >= atol(mysqlchar_array[fd][5]))
  {
    logmessage("rcpt=%lu, throttle=abuse, host=%s, from=%s, to=%s, size=%d/%d, quota=%d/%d, count=%d/%d(%d)\n",
      rcpt_count,                       /* recipient count      */
      host_array[fd][2],                /* host                 */
      triplet_array[fd][1],             /* from                 */
      triplet_array[fd][2],             /* to                   */
      atol(triplet_array[fd][3]),       /* size_cur             */
      atol(mysqlchar_array[fd][7]),     /* size_max             */
      atol(mysqlchar_array[fd][4]),     /* quota_cur            */
      atol(mysqlchar_array[fd][5]),     /* quota_max            */
      atol(mysqlchar_array[fd][2]),	/* count_cur		*/
      atol(mysqlchar_array[fd][1]),     /* count_max            */
      atol(mysqlchar_array[fd][8])      /* count_tot            */
    );
      
    return (-5);
  }

  /* if the sender has not reached his quota, increase count */
  if(atol(mysqlchar_array[fd][2]) < atol(mysqlchar_array[fd][1]))
  {
    logmessage("rcpt=%lu, throttle=update, host=%s, from=%s, to=%s, size=%d/%d, quota=%d/%d, count=%d/%d(%d)\n",
      rcpt_count,                       /* recipient count      */
      host_array[fd][2],                /* host                 */
      triplet_array[fd][1],             /* from                 */
      triplet_array[fd][2],             /* to                   */
      atol(triplet_array[fd][3]),       /* size_cur             */
      atol(mysqlchar_array[fd][7]),     /* size_max             */
      atol(mysqlchar_array[fd][4])+atol(triplet_array[fd][3]), /* quota_cur */
      atol(mysqlchar_array[fd][5]),     /* quota_max            */
      atol(mysqlchar_array[fd][2])+1,	/* count_cur		*/
      atol(mysqlchar_array[fd][1]),     /* count_max            */
      atol(mysqlchar_array[fd][8])      /* count_tot            */
    );
      
    /* build up & execute query */
    snprintf(mysqlquery_array[fd], 512,
      "UPDATE throttle SET _count_cur=_count_cur+1,_count_tot=_count_tot+1,_quota_cur=_quota_cur+%ld WHERE _from='%s'",
      atol(triplet_array[fd][3]), host_array[fd][2]);
    if(db_doquery(fd) == -1) return(db_failure(fd, "throttle"));
  }

  return (0);
}
 
/* EOF */
